#!/usr/bin/env python2.3
#
#   mibcip - Modular IRC Bot Core In Python
#   Copyright (C) 2004  Helmut Grohne
#
#   This program is free software; you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation; either version 2 of the License, or
#   (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program; if not, write to the Free Software
#   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#

conflicts = []
depends = [ ["core.event"], ["core.rawparser"], ["core.parser"], ["core.info"],
            ["core.auth"] ]

class plugin:
    def __init__(self, base):
        self.base = base
        # map nicknames to True (registered nick) or False
        self.cache = {}
        # map nicknames to update state (True during update)
        self.updates = {}
        # map nicknames to arguments
        self.arguments = {}

        core = base.p.core
        core.auth.register_method("nickserv", self.auth_nickserv)
        attach_event = base.p.core.event.attach_event
        attach_event("core.rawparser", "auth.nickserv", "318", self.on_318)
        attach_event("core.rawparser", "auth.nickserv", "320", self.on_nickserv)
        attach_event("core.rawparser", "auth.nickserv", "307", self.on_nickserv)
        attach_event("core.info", "auth.nickserv", "NICKLOST", self.on_nicklost)
        attach_event("core.parser", "auth.nickserv", "NICK", self.on_nick)

    def __getstate__(self):
        return self.cache, self.updates, self.arguments

    def __setstate__(self, (cache, updates, arguments)):
        self.cache = cache
        self.updates = updates
        self.arguments = arguments

    def auth_nickserv(self, nick, arg):
        self.arguments[nick] = arg
        core = self.base.p.core
        if nick in self.cache:
            core.auth.report("nickserv", arg, nick, self.cache[nick])
        else:
            if nick not in self.updates:
                self.updates[nick] = True
                core.cmd.whois(nick)

    def update_cache(self, nick, value):
        if value or nick in self.updates or nick not in self.cache:
            self.cache[nick] = value
        try:
            del self.updates[nick]
        except KeyError:
            pass
        if nick in self.arguments:
            self.base.p.core.auth.report("nickserv", self.arguments[nick],
                                         nick, self.cache[nick])

    def on_nickserv(self, (arg,)):
        if arg.text not in ('is an identified user', 'is a registered nick', 'is identified to services '):
            return
        self.update_cache(arg.argv[1], True)

    def on_318(self, (arg,)):
        """RPL_ENDOFWHOIS"""
        self.update_cache(arg.argv[1], False)

    def on_nicklost(self, (nick,)):
        try:
            del self.cache[nick]
        except KeyError:
            pass
        try:
            del self.updates[nick]
        except KeyError:
            pass
        try:
            del self.arguments[nick]
        except KeyError:
            pass

    def on_nick(self, (nick, new)):
        if nick in self.cache:
            self.cache[new] = self.cache[nick]
            del self.cache[nick]
        if nick in self.updates:
            self.updates[new] = self.updates[nick]
            del self.updates[nick]
        if nick in self.arguments:
            self.arguments[new] = self.arguments[nick]
            del self.arguments[nick]
