#!/usr/bin/env python2.3
#
#   mibcip - Modular IRC Bot Core In Python
#   Copyright (C) 2004  Helmut Grohne
#
#   This program is free software; you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation; either version 2 of the License, or
#   (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program; if not, write to the Free Software
#   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#


conflicts = []
depends = [ ["core.send"], ["core.info"], ["core.event"], ["core.servcapab"] ]

class plugin:
    def __init__(self, base):
        self.basep = base.p
        base.p.core.event.register_plugin("core.cmd")

    def __getstate__(self):
        """This is method is to reconstruct the instance. Do not use with
        pickle."""
        return None

    def __setstate__(self, state):
        """This is method is to reconstruct the instance. Do not use with
        pickle."""

    def mynick(self):
        """Return my nickname. Also provided by core.info."""
        return self.basep.core.info.myinf.nick

    def emit_signal(self, name, nick, target, text):
        """Emit a signal via event handler. Don't call this method."""
        arg = (nick, target, text)
        self.basep.core.event.call_event("core.cmd", name, arg)

    def quote(self, command, prio=None, force=False):
        """Send command with *args joined together with delimiter join with
        given priority and ignored force flag."""

        send = self.basep.core.send

        if prio is None:
            prio = "NRM"

        if isinstance(prio, str):
            prio = getattr(send, prio)

        send.quote(command, prio)

    def ctcprep(self, target, command, prio=None, force=False):
        """Send a ctcp reply to given target."""
        command = "\x01%s\x01" % command
        return self.notice(target, command, prio=prio, force=force)

    def ctcpreq(self, target, command, prio=None, force=False):
        """Send a ctcp request to given target."""
        command = "\x01%s\x01" % command
        return self.privmsg(target, command, prio=prio, force=force)

    def join(self, channel, key=None, prio=None, force=False):
        """Make the bot join a number of channels (channel) with optional keys
        (key). Both channel and key may be a lists or strings. key must not
        contain more elements or more ',' than channel. If force is set to True
        don't filter out already joined channels and perhaps don't send the
        command at all. The prio argument can change the priority (see
        core.send for details)."""
        if not force:
            if isinstance(channel, str):
                channel = channel.split(",")
                if key is not None:
                    key = key.split(",")
            else:
                channel = list(channel)
                if key is not None:
                    key = list(key)

            if key is None:
                key = []

            # Make key as long as channel.
            key.extend([None] * (len(channel) - len(key)))

            # Make a mapping from channel names to keys.
            channel = dict(zip(channel, key))

            # Remove already joined channels.
            for chan in self.basep.core.info.chans.keys():
                try:
                    del channel[chan]
                except KeyError:
                    pass

            channel = channel.items()

            # Make sure channels without keys come last:
            def compare_function((channel_a, key_a), (channel_b, key_b)):
                # 1 if None, 0 unless.
                a = int(key_a is None)
                b = int(key_b is None)
                # Ones will be last in the result.
                return cmp(a, b)
            channel.sort(compare_function)

            # Join needed keys with ','.
            key = ",".join([v for k, v in channel if v is not None])
            # Join all channels with ','.
            channel = ",".join([k for k, v in channel])

            if not channel:
                return
        else:
            if not isinstance(channel, str):
                channel = ",".join(list(channel))
            if key is not None and not isinstance(key, str):
                key = ",".join(key)

        # key may be str or None.
        if key:
            key = " " + key
        else:
            key = ""

        if prio is None:
            prio = "IMP"

        self.quote("JOIN %s%s" % (channel, key), prio=prio)

    def nick(self, newnick, prio=None, force=False):
        """Change my nick to newick with given priority. The force flag may be
        used to change my nick to a nick i already have."""

        if newnick == self.mynick() and not force:
            return

        if prio is None:
            prio = "URG"

        self.quote("NICK %s" % newnick, prio=prio)

    def notice(self, target, msg, prio=None, force=False):
        """Send a notice to targed with given priority and ignored force
        flag."""

        self.quote("NOTICE %s :%s" % (target, msg), prio=prio)

        mynick = self.mynick()

        self.emit_signal("NOTICE", mynick, target, msg)

        if len(msg) > 2 and msg[0] == '\x01' and msg[-1] == '\x01':
            self.emit_signal("CTCPREP", mynick, target, msg[1:-1])
        elif self.basep.core.servcapab.ischannel(target):
            self.emit_signal("CNOTICE", mynick, target, msg)
        else:
            self.emit_signal("PNOTICE", mynick, target, msg)

    def part(self, channel, prio=None, force=False):
        """Part given channels (as list or string, see join) with given
        priority. If the force flag is not set, omit channels that are not
        joined and perhaps the complete part message."""
        if not force:
            if isinstance(channel, str):
                channel = channel.split(",")
            else:
                channel = list(channel)

            joined_channels = self.basep.core.info.chans
            channel = [c for c in channel if c not in joined_channels]
            if not channel:
                return

        if not isinstance(channel, str):
            channel = ",".join(channel)

        if prio is None:
            prio = "IMP"

        self.quote("PART %s" % channel, prio=prio)

    def ping(self, target, prio=None, force=False):
        """Send a ping to target with optional priority and ignorec force
        flag."""
        if prio is None:
            prio = "IMP"

        self.quote("PING %s" % target, prio=prio)

    def privmsg(self, target, msg, prio=None, force=False):
        """Send a privmsg to targed with given priority and ignored force
        flag."""

        self.quote("PRIVMSG %s :%s" % (target, msg), prio=prio)

        mynick = self.mynick()

        self.emit_signal("PRIVMSG", mynick, target, msg)

        if len(msg) > 2 and msg[0] == '\x01' and msg[-1] == '\x01':
            self.emit_signal("CTCPREQ", mynick, target, msg[1:-1])
        elif self.basep.core.servcapab.ischannel(target):
            self.emit_signal("CPRIVMSG", mynick, target, msg)
        else:
            self.emit_signal("PPRIVMSG", mynick, target, msg)

    def quit(self, reason, prio=None, force=False):
        """Quit the server with given reason, priority and ignored force
        flag."""

        if prio is None:
            prio = "URG"

        self.quote("QUIT :%s" % reason, prio=prio)

    def time(self, prio=None, force=False):
        """Request the server time from the server with given priority and
        ignored force flag."""

        if prio is None:
            prio = "URG"

        self.quote("TIME", prio=prio)

    def user(self, user, host, server, realname, prio=None, force=False):
        """Introduce me to the server with given user, host, server, realname,
        optional priority and ignored force flag."""

        if prio is None:
            prio = "URG"

        command = "USER %s %s %s :%s" % (user, host, server, realname)
        self.quote(command, prio=prio)

    def whois(self, nick, prio=None, force=False):
        """Request information about given nickname with optional priority and
        ignored force flag."""

        self.quote("WHOIS %s" % nick, prio=prio)
