#!/usr/bin/env python2.3
#
#   mibcip - Modular IRC Bot Core In Python
#   Copyright (C) 2004  Helmut Grohne
#
#   This program is free software; you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation; either version 2 of the License, or
#   (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program; if not, write to the Free Software
#   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#

conflicts = []
depends = [ ["core.event"], ["core.send"] ]

class IrcProtocolError(Exception):
    pass

class ircParser:
    """This class provides a structure to access parts of a parsed line.
    line = the whole line, nick = nick if given, user = user if given,
    host = host if given, cmd = command.upper(), argv = list of arguments,
    text = everything behind the final :"""
    def parse(self, line):
        """Fill data structur based on contents of line."""
        self.line = line
        if self.line[0] == ':':           # if line is like :nick!user@host ..
            rest = self.line[1:]          # rest="nick!user@host ..."

            tlst = rest.split(' ', 1)     # tlst=["nick!user@host","..."]
            try:
                tmp, rest = tlst
            except ValueError:
                raise IrcProtocolError

            tlst = tmp.split('!', 1)      # tlst=["nick","user@host"]
            try:
                self.nick, tmp = tlst
            except ValueError:
                self.nick ,= tlst
                tmp = ""

            tlst = tmp.split('@', 1)      # tlst=["user","host"]
            try:
                self.user, self.host = tlst
            except ValueError:
                self.user ,= tlst
                self.host = ""

        else:                             # line isn't like :nick!user@host ...
            self.nick = self.user = self.host = ""
            rest = self.line

        tlst = rest.split(' ', 1)         # tlst=["CMD","a r g v :text"]
        try:
            self.cmd, rest = tlst
        except ValueError:
            raise IrcProtocolError

        self.cmd = self.cmd.upper()       # cmd is all upcase

        rest = " " + rest.rstrip("\n\r")  # rest=" [a r g v ]:text"

        tlst = rest.split(' :', 1)        # tlst=["[ a r g v]","text"]
        try:
            rest, self.text = tlst
        except ValueError:
            self.text = ""

        # argv = ["a","r","g","v"]
        self.argv = filter(None, rest.split(' '))

    def __repr__(self):
        argv = "|".join(self.argv)
        insert = (self.nick, self.user, self.host, self.cmd, argv, self.text)
        return ":[%s]![%s]@[%s] [%s] [%s] :[%s]" % insert

class plugin:
    def __init__(self, base):
        self.base = base
        self.buff = ""

        event_handler = self.base.p.core.event
        event_handler.register_plugin("core.rawparser")
        event_handler.attach_event("core.event", "core.rawparser",
                                   "data", self.receive_data)

    def __getstate__(self):
        return self.buff

    def __setstate__(self, state):
        self.buff = state

    def receive_data(self, data):
        """Process a given amount of data and call receive_line for each
        complete line."""
        lines = (self.buff + data[0]).split("\n")
        self.buff = lines.pop()
        for l in lines:
            self.receive_line(l.rstrip("\r"))

    def receive_line(self, line):
        """Process a single line, answer PINGs and make the event handler call
        an event based on the command name."""

        parsed = ircParser()
        try:
            parsed.parse(line)
        except IrcProtocolError:
            return

        core = self.base.p.core

        print repr(parsed)

        if parsed.cmd == "PING":
            pos = parsed.line.upper().find("PING")
            l = "%sPONG%s" % (parsed.line[:pos], parsed.line[pos+4:])
            core.send.quote(l, core.send.URG)
        else:
            core.event.call_event("core.rawparser", parsed.cmd, (parsed,))
