#!/usr/bin/env python2.3
#
#   mibcip - Modular IRC Bot Core In Python
#   Copyright (C) 2004  Helmut Grohne
#
#   This program is free software; you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation; either version 2 of the License, or
#   (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program; if not, write to the Free Software
#   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#

try:
    #raise ImportError
    from twisted.internet import reactor
except ImportError:
    from twisted_emu import reactor


conflicts = []
depends = [ ["core.event"], ["core.config"] ]

class plugin:
    pingfreq = 300
    timeout = 60
    reconwait = 300

    def __init__(self, base):
        self.base = base
        self.retry = True
        # TiMeoutDelayedCall
        self.tmdc = None
        attach_event = base.p.core.event.attach_event
        attach_event("core.event", "core.reconnector", "disconnect",
                     self.on_disconnect)
        attach_event("core.event", "core.reconnector", "connect",
                     self.update_timeout)
        attach_event("core.event", "core.reconnector", "data",
                     self.update_timeout)
        attach_event("core.event", "core.reconnector", "confailed",
                     self.on_confailed)

    def __getstate__(self):
        return self.retry, self.tmdc

    def __setstate__(self, (retry, tmdc)):
        self.retry = retry
        self.tmdc = tmdc

    def quit(self):
        """Stop reconnecting."""
        self.retry = False

    def wait_timeout(self):
        """This method is called by reactor.callLater to remove close the
        connection as no pong reply was received."""
        self.tmdc = None
        p = self.base.protoinstance
        if p:
            p.transport.loseConnection()

    def send_ping(self):
        """This method is called by reactor.callLater to send a ping as the
        connection was inactive for a specific amount of time."""
        self.tmdc = None
        self.base.p.core.cmd.ping("bork")
        self.tmdc = reactor.callLater(self.timeout, self.wait_timeout)

    def update_timeout(self, spam=None):
        """Marks the connection as active by resetting current timeouts."""
        if self.tmdc:
            self.tmdc.cancel()
        self.tmdc = reactor.callLater(self.pingfreq, self.send_ping)

    def do_connect(self):
        """Connect now, used internally."""
        if not self.retry:
            return

        config = self.base.p.core.config
        server = config.get("core", "server")
        port = config.getint("core", "port")

        reactor.connectTCP(server, port, self.base)

    def on_disconnect(self, arg):
        """Automatically reconnect."""
        self.do_connect()

    def on_confailed(self, arg):
        """Automatically reconnect after waiting some time."""
        if not self.retry:
            return

        reactor.callLater(self.reconwait, self.do_connect)
