#!/usr/bin/env python2.3
#
#   mibcip - Modular IRC Bot Core In Python
#   Copyright (C) 2004  Helmut Grohne
#
#   This program is free software; you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation; either version 2 of the License, or
#   (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program; if not, write to the Free Software
#   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#

import time

try:
    #raise ImportError
    from twisted.internet import reactor
except ImportError:
    from twisted_emu import reactor


conflicts = []
depends = [ ["core.event"] ]

class plugin:
    URG = 0 # urgent
    IMP = 8 # important
    NRM = 16 # text
    IDL = 24 # idle
    terminator = "\r\n"
    sec_ahead = 8
    sec_permsg = 2
    sec_bhead = 2
    sec_perbyte = 1/float(64)

    def __init__(self, base):
        self.base = base
        self.later = None
        self.queue = []
        self.msgtime = 0
        self.bytetime = 0
        event_handler = base.p.core.event
        event_handler.register_plugin("core.send")

    def __getstate__(self):
        return (self.queue, self.msgtime, self.bytetime, self.later)

    def __setstate__(self, state):
        self.queue, self.msgtime, self.bytetime, self.later = state

    def emit_signal(self, data):
        """Emit signal to let other plugins know about outgoing data."""
        self.base.p.core.event.call_event("core.send", "data", (data,))

    def wait_send(self, t=None):
        """Compute how much seconds to wait before next send."""
        if t is None:
            t = time.time()
        # self.msgtime should be smaller than t + self.sec_ahead.
        # If it is larger we have to wait that amount before sending next.
        message_wait = self.msgtime - (t + self.sec_ahead)
        byte_wait = self.bytetime - (t + self.sec_bhead)
        return max(0, message_wait, byte_wait)

    def check_send(self, t=None):
        """Check if sending is possible without waiting."""
        if t is None:
            t = time.time()
        if not self.msgtime < t + self.sec_ahead:
            return False
        if not self.bytetime < t + self.sec_bhead:
            return False
        return True

    def do_send(self):
        """Send as much lines as possible without waiting and install a handler
        with reactor.callLater if the queue didn't empty."""
        self.later = None

        if not self.queue:
            return

        t = time.time()

        while self.queue and self.check_send(t):
           sent_length = self.real_send()

           if self.msgtime < t:
               self.msgtime = t
           self.msgtime += self.sec_permsg

           if self.bytetime < t:
               self.bytetime = t
           self.bytetime += sent_length * self.sec_perbyte

        if self.queue:
            wait = self.wait_send(t)
            self.later = reactor.callLater(wait, self.do_send)

    def real_send(self):
        """Actually send the next line."""
        msg = self.queue.pop(0)[1]
        print "<-" + msg # DEBUG
        self.base.protoinstance.transport.write(msg)
        self.emit_signal(msg)
        return len(msg)

    def rawquote(self, msg, lvl):
        """Append msg to the queue and try to empty it without waiting and try
        sending."""
        start = 0
        end = len(self.queue)
        while start < end:
            mid = (start+end)/2
            if self.queue[mid][0] > lvl:
                end = mid
            else:
                start = mid+1

        self.queue.insert(start, (lvl,msg))

        if not self.later:
            self.do_send()

    def quote(self, msg, lvl):
        """Append terminated msg to queue using rawquote and try sending."""
        self.rawquote(msg + self.terminator, lvl)
