#!/usr/bin/env python2.3
#
#   mibcip - Modular IRC Bot Core In Python
#   Copyright (C) 2004  Helmut Grohne
#
#   This program is free software; you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation; either version 2 of the License, or
#   (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program; if not, write to the Free Software
#   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#

conflicts = []
depends = [ ["core.config"] ]

class basicCapab:
    """Basic server capabilities derieve this and redefine config values for
    new servers."""

    # all channel user mode characters supported by server in descending order.
    cumods = "ov"
    # cumods = ohv ; where? # TODO
    # corresponding prefixes. must be same length as cumod.
    cupres = "@+"
    # cupres = @%+ ; where? # TODO
    # all single modes (without arguments) supported by server
    smods = "imnrst"
    # the modes with arguments, whose arguments have to be added on removal
    abmods = "k"
    # the modes with arguments, whose arguments are only present on addition
    aamods = "l"
    # user modes
    umods = "iw"
    # channel prefixes
    chpres = "#"

    def __init__(self):
        self.amods = self.aamods + self.abmods

    def ischannel(self, name):
        """Checks if given name corresponds to a channel."""
        return name[0] in self.chpres

    def cumodeprefix(self, mode):
        """Find the prefix corresponding to given channel user mode."""
        if mode in self.cumods:
            return self.cupres[self.cumods.find(mode)]
        return ''

    def insertprefix(self, prefixes, new):
        """Insert given prefix into a a str of prefixes in order of power."""
        if new in prefixes:
            return prefixes
        t = list(prefixes)
        t.append(new)
        t.sort(lambda a,b: cmp(self.cupres.find(a), self.cupres.find(b)))
        return "".join(t)

    def removeprefix(self, prefixes, old):
        """Remove given prefix from a str of prefixes."""
        return prefixes.replace(old, '')

class euircCapab(basicCapab):
    """Capabilities of the servers used in euIRC."""
    cumods = "qaohv"
    cupres = "*!@%+"
    smods = "Scimnrst"
    umods = "iwx"

class freenodeCapab(basicCapab):
    """Capabilities of dancerircds used by Freenode."""
    smods = "cimnrRst"
    umods = "iew"

class quakenetCapab(basicCapab):
   """Capabilities of the servers used in Quakenet."""
   smods = "imnpstrDdcCNu"
   umods = "iwhrfBIdGvgskSaANTCOoxeWqHz"

capabs = { "basic":basicCapab, "euirc":euircCapab, "freenode":freenodeCapab,
        "quakenet":quakenetCapab }

class plugin:
    def __init__(self, base):
        self.name = "basic"
        self.config = basicCapab()
        self.settype(base.p.core.config.get("core.servcapab", "config"))

    def __getstate__(self):
        return self.name

    def __setstate__(self, state):
        self.settype(state)

    def __getattr__(self, attr):
        """Map attributes to capabiltiy class."""
        return getattr(self.config, attr)

    def settype(self, name):
        """Set server type to given type. Name must exist in capabs dict."""
        self.name = name
        self.config = capabs[name]()
