#!/usr/bin/env python2.3
#
#   mibcip - Modular IRC Bot Core In Python
#   Copyright (C) 2004  Helmut Grohne
#
#   This program is free software; you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation; either version 2 of the License, or
#   (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program; if not, write to the Free Software
#   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#

conflicts = []
depends = [ ["core.config"], ["core.event"], ["core.parser"], ["core.info"] ]

import time

def time_str():
    return time.strftime("%H:%M", time.localtime())

class plugin:
    events = "addban amode cumode invite join nick notice part privmsg kick"
    events += " quit smode topic unban"
    events = events.split()
    def __init__(self, base):
        self.base = base
        self.filename = base.p.core.config.get("test.log", "filename")
        self.logfile = file(self.filename, "a")

        attach_event = base.p.core.event.attach_event
        for e in self.events:
            method = getattr(self, "on_%s" % e)
            attach_event("core.parser", "test.log", e.upper(), method)

        for e in ("notice", "privmsg"):
            method = getattr(self, "on_%s" % e)
            attach_event("core.cmd", "test.log", e.upper(), method)

    def __getstate__(self):
        return None

    def __setstate__(self, state):
        pass

    def writelog(self, msg):
        self.logfile.write("%s %s\n" % (time_str(), msg))
        self.logfile.flush()

    def on_addban(self, (nick, chan, mask)):
        self.writelog("--- %s bans %s from %s" % (nick, mask, chan))

    def on_amode(self, (nick, chan, mode, arg)):
        self.writelog("--- %s sets mode %s %s on %s" % (nick, mode, arg, chan))

    def on_cumode(self, (nick, chan, mode, target)):
        insert = (nick, mode, target, chan)
        self.writelog("--- %s sets mode %s %s on %s" % insert)

    def on_invite(self, (nick, target)):
        self.writelog("--- %s invites me to %s" % (nick, target))

    def on_join(self, (nick, chan)):
        self.writelog("--- %s has joined %s" % (nick, chan))

    def on_kick(self, (nick, chan, target, reason)):
        insert = (target, chan, nick, reason)
        self.writelog("--- %s was kicked from %s by %s [%s]" % insert)

    def on_nick(self, (old, new)):
        self.writelog("--- %s is now known as %s" % (old, new))

    def on_notice(self, (nick, target, text)):
        chans = self.base.p.core.info.chans
        if target in chans:
            channel = chans[target]
            if channel.hasNick(nick):
                nick = channel.users[nick][0:1] + nick
        self.writelog("%s: [%s] %s" % (target, nick, text))

    def on_part(self, (nick, chan, reason)):
        self.writelog("--- %s has left %s [%s]" % (nick, chan, reason))

    def on_privmsg(self, (nick, target, text)):
        chans = self.base.p.core.info.chans
        if target in chans:
            channel = chans[target]
            if channel.hasNick(nick):
                nick = channel.users[nick][0:1] + nick
        self.writelog("%s: <%s> %s" % (target, nick, text))

    def on_quit(self, (nick, reason)):
        self.writelog("--- %s has quit [%s]" % (nick, reason))

    def on_smode(self, (nick, chan, mode)):
        self.writelog("--- %s sets mode %s on %s" % (nick, mode, chan))

    def on_topic(self, (nick, chan, topic)):
        self.writelog("--- %s sets topic on %s to %s" % (nick, chan, topic))

    def on_unban(self, (nick, channel, mask)):
        self.writelog("--- %s unbans %s on %s" % (nick, mask, channel))

"""
Missing:
- SETNICK (nick,) = set my nickname to nick
- SETMODES (chan, dct) = initialize channel modes for channel chan with dct
- SETTOPIC (chan, topic)
- SETPCSRC (chan, who, when)
- ADDWHO (nick, user, host, realname)
- NAMES (chan, names) = userdict contains nicks as keys and prefixes as values
- SETBAN (chan, mask, srcmask, time) = time is seconds since epoch or None for
    now
- SETTIME (time,)
- UMODE (mode) = single user mode change
- CTCPREP (nick, target, text)
- CTCPREQ (nick, target, text)
"""
